@@ -132,20 +132,19 @@ backwards compatibility. At first chance, set ``object_post_as_copy`` to
 """
 
 import os
-from urllib import quote
-from ConfigParser import ConfigParser, NoSectionError, NoOptionError
-from six.moves.urllib.parse import unquote
+from six.moves.configparser import ConfigParser, NoSectionError, NoOptionError
+from six.moves.urllib.parse import quote, unquote
 
 from swift.common import utils
 from swift.common.utils import get_logger, \
     config_true_value, FileLikeIter, read_conf_dir, close_if_possible
 from swift.common.swob import Request, HTTPPreconditionFailed, \
-    HTTPRequestEntityTooLarge, HTTPBadRequest
+    HTTPRequestEntityTooLarge, HTTPBadRequest, HTTPException
 from swift.common.http import HTTP_MULTIPLE_CHOICES, HTTP_CREATED, \
-    is_success
+    is_success, HTTP_OK
 from swift.common.constraints import check_account_format, MAX_FILE_SIZE
 from swift.common.request_helpers import copy_header_subset, remove_items, \
-    is_sys_meta, is_sys_or_user_meta
+    is_sys_meta, is_sys_or_user_meta, is_object_transient_sysmeta
 from swift.common.wsgi import WSGIContext, make_subrequest
 
 
@@ -206,16 +205,18 @@ def _check_destination_header(req):
                               '<container name>/<object name>')
 
 
-def _copy_headers_into(from_r, to_r):
+def _copy_headers(src, dest):
     """
-    Will copy desired headers from from_r to to_r
-    :params from_r: a swob Request or Response
-    :params to_r: a swob Request or Response
+    Will copy desired headers from src to dest.
+
+    :params src: an instance of collections.Mapping
+    :params dest: an instance of collections.Mapping
     """
-    pass_headers = ['x-delete-at']
-    for k, v in from_r.headers.items():
-        if is_sys_or_user_meta('object', k) or k.lower() in pass_headers:
-            to_r.headers[k] = v
+    for k, v in src.items():
+        if (is_sys_or_user_meta('object', k) or
+                is_object_transient_sysmeta(k) or
+                k.lower() == 'x-delete-at'):
+            dest[k] = v
 
 
 class ServerSideCopyWebContext(WSGIContext):
@@ -322,16 +323,20 @@ class ServerSideCopyMiddleware(object):
         # the client actually sent.
         req.environ['swift.orig_req_method'] = req.method
 
-        if req.method == 'PUT' and req.headers.get('X-Copy-From'):
-            return self.handle_PUT(req, start_response)
-        elif req.method == 'COPY':
-            return self.handle_COPY(req, start_response)
-        elif req.method == 'POST' and self.object_post_as_copy:
-            return self.handle_object_post_as_copy(req, start_response)
-        elif req.method == 'OPTIONS':
-            # Does not interfere with OPTIONS response from (account,container)
-            # servers and /info response.
-            return self.handle_OPTIONS(req, start_response)
+        try:
+            if req.method == 'PUT' and req.headers.get('X-Copy-From'):
+                return self.handle_PUT(req, start_response)
+            elif req.method == 'COPY':
+                return self.handle_COPY(req, start_response)
+            elif req.method == 'POST' and self.object_post_as_copy:
+                return self.handle_object_post_as_copy(req, start_response)
+            elif req.method == 'OPTIONS':
+                # Does not interfere with OPTIONS response from
+                # (account,container) servers and /info response.
+                return self.handle_OPTIONS(req, start_response)
+
+        except HTTPException as e:
+            return e(req.environ, start_response)
 
         return self.app(env, start_response)
 
@@ -422,9 +427,7 @@ class ServerSideCopyMiddleware(object):
                     source_resp.headers['last-modified']
         # Existing sys and user meta of source object is added to response
         # headers in addition to the new ones.
-        for k, v in sink_req.headers.items():
-            if is_sys_or_user_meta('object', k) or k.lower() == 'x-delete-at':
-                resp_headers[k] = v
+        _copy_headers(sink_req.headers, resp_headers)
         return resp_headers
 
     def handle_PUT(self, req, start_response):
@@ -456,10 +459,33 @@ class ServerSideCopyMiddleware(object):
             close_if_possible(source_resp.app_iter)
             return source_resp(source_resp.environ, start_response)
 
-        # Create a new Request object based on the original req instance.
-        # This will preserve env and headers.
-        sink_req = Request.blank(req.path_info,
-                                 environ=req.environ, headers=req.headers)
+        # Create a new Request object based on the original request instance.
+        # This will preserve original request environ including headers.
+        sink_req = Request.blank(req.path_info, environ=req.environ)
+
+        def is_object_sysmeta(k):
+            return is_sys_meta('object', k)
+
+        if 'swift.post_as_copy' in sink_req.environ:
+            # Post-as-copy: ignore new sysmeta, copy existing sysmeta
+            remove_items(sink_req.headers, is_object_sysmeta)
+            copy_header_subset(source_resp, sink_req, is_object_sysmeta)
+        elif config_true_value(req.headers.get('x-fresh-metadata', 'false')):
+            # x-fresh-metadata only applies to copy, not post-as-copy: ignore
+            # existing user metadata, update existing sysmeta with new
+            copy_header_subset(source_resp, sink_req, is_object_sysmeta)
+            copy_header_subset(req, sink_req, is_object_sysmeta)
+        else:
+            # First copy existing sysmeta, user meta and other headers from the
+            # source to the sink, apart from headers that are conditionally
+            # copied below and timestamps.
+            exclude_headers = ('x-static-large-object', 'x-object-manifest',
+                               'etag', 'content-type', 'x-timestamp',
+                               'x-backend-timestamp')
+            copy_header_subset(source_resp, sink_req,
+                               lambda k: k.lower() not in exclude_headers)
+            # now update with original req headers
+            sink_req.headers.update(req.headers)
 
         params = sink_req.params
         if params.get('multipart-manifest') == 'get':
@@ -467,38 +493,43 @@ class ServerSideCopyMiddleware(object):
                 params['multipart-manifest'] = 'put'
             if 'X-Object-Manifest' in source_resp.headers:
                 del params['multipart-manifest']
-                sink_req.headers['X-Object-Manifest'] = \
-                    source_resp.headers['X-Object-Manifest']
+                if 'swift.post_as_copy' not in sink_req.environ:
+                    sink_req.headers['X-Object-Manifest'] = \
+                        source_resp.headers['X-Object-Manifest']
             sink_req.params = params
 
         # Set data source, content length and etag for the PUT request
         sink_req.environ['wsgi.input'] = FileLikeIter(source_resp.app_iter)
         sink_req.content_length = source_resp.content_length
-        sink_req.etag = source_resp.etag
+        if (source_resp.status_int == HTTP_OK and
+                'X-Static-Large-Object' not in source_resp.headers and
+                ('X-Object-Manifest' not in source_resp.headers or
+                 req.params.get('multipart-manifest') == 'get')):
+            # copy source etag so that copied content is verified, unless:
+            #  - not a 200 OK response: source etag may not match the actual
+            #    content, for example with a 206 Partial Content response to a
+            #    ranged request
+            #  - SLO manifest: etag cannot be specified in manifest PUT; SLO
+            #    generates its own etag value which may differ from source
+            #  - SLO: etag in SLO response is not hash of actual content
+            #  - DLO: etag in DLO response is not hash of actual content
+            sink_req.headers['Etag'] = source_resp.etag
+        else:
+            # since we're not copying the source etag, make sure that any
+            # container update override values are not copied.
+            remove_items(sink_req.headers, lambda k: k.startswith(
+                'X-Object-Sysmeta-Container-Update-Override-'))
 
         # We no longer need these headers
         sink_req.headers.pop('X-Copy-From', None)
         sink_req.headers.pop('X-Copy-From-Account', None)
+
         # If the copy request does not explicitly override content-type,
         # use the one present in the source object.
         if not req.headers.get('content-type'):
             sink_req.headers['Content-Type'] = \
                 source_resp.headers['Content-Type']
 
-        fresh_meta_flag = config_true_value(
-            sink_req.headers.get('x-fresh-metadata', 'false'))
-
-        if fresh_meta_flag or 'swift.post_as_copy' in sink_req.environ:
-            # Post-as-copy: ignore new sysmeta, copy existing sysmeta
-            condition = lambda k: is_sys_meta('object', k)
-            remove_items(sink_req.headers, condition)
-            copy_header_subset(source_resp, sink_req, condition)
-        else:
-            # Copy/update existing sysmeta and user meta
-            _copy_headers_into(source_resp, sink_req)
-            # Copy/update new metadata provided in request if any
-            _copy_headers_into(req, sink_req)
-
         # Create response headers for PUT response
         resp_headers = self._create_response_headers(source_path,
                                                      source_resp, sink_req)
