@@ -11,7 +11,7 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
+import json
 import uuid
 import random
 from nose import SkipTest
@@ -51,35 +51,47 @@ class TestContainerSync(ReplProbeTest):
         super(TestContainerSync, self).setUp()
         self.realm, self.cluster = get_current_realm_cluster(self.url)
 
-    def _setup_synced_containers(self, skey='secret', dkey='secret'):
+    def _setup_synced_containers(
+            self, source_overrides=None, dest_overrides=None):
+        # these defaults are used to create both source and dest containers
+        # unless overridden by source_overrides and/or dest_overrides
+        default_params = {'url': self.url,
+                          'token': self.token,
+                          'account': self.account,
+                          'sync_key': 'secret'}
+
         # setup dest container
-        dest_container = 'dest-container-%s' % uuid.uuid4()
+        dest = dict(default_params)
+        dest['name'] = 'dest-container-%s' % uuid.uuid4()
+        dest.update(dest_overrides or {})
         dest_headers = {}
         dest_policy = None
         if len(ENABLED_POLICIES) > 1:
             dest_policy = random.choice(ENABLED_POLICIES)
             dest_headers['X-Storage-Policy'] = dest_policy.name
-        if dkey is not None:
-            dest_headers['X-Container-Sync-Key'] = dkey
-        client.put_container(self.url, self.token, dest_container,
+        if dest['sync_key'] is not None:
+            dest_headers['X-Container-Sync-Key'] = dest['sync_key']
+        client.put_container(dest['url'], dest['token'], dest['name'],
                              headers=dest_headers)
 
         # setup source container
-        source_container = 'source-container-%s' % uuid.uuid4()
+        source = dict(default_params)
+        source['name'] = 'source-container-%s' % uuid.uuid4()
+        source.update(source_overrides or {})
         source_headers = {}
-        sync_to = '//%s/%s/%s/%s' % (self.realm, self.cluster, self.account,
-                                     dest_container)
+        sync_to = '//%s/%s/%s/%s' % (self.realm, self.cluster, dest['account'],
+                                     dest['name'])
         source_headers['X-Container-Sync-To'] = sync_to
-        if skey is not None:
-            source_headers['X-Container-Sync-Key'] = skey
+        if source['sync_key'] is not None:
+            source_headers['X-Container-Sync-Key'] = source['sync_key']
         if dest_policy:
             source_policy = random.choice([p for p in ENABLED_POLICIES
                                            if p is not dest_policy])
             source_headers['X-Storage-Policy'] = source_policy.name
-        client.put_container(self.url, self.token, source_container,
+        client.put_container(source['url'], source['token'], source['name'],
                              headers=source_headers)
 
-        return source_container, dest_container
+        return source['name'], dest['name']
 
     def _test_sync(self, object_post_as_copy):
         source_container, dest_container = self._setup_synced_containers()
@@ -148,10 +160,97 @@ class TestContainerSync(ReplProbeTest):
     def test_sync_with_fast_post(self):
         self._test_sync(False)
 
+    def test_sync_slo_manifest(self):
+        # Verify that SLO manifests are sync'd even if their segments can not
+        # be found in the destination account at time of sync'ing.
+        # Create source and dest containers for manifest in separate accounts.
+        dest_account = self.account_2
+        source_container, dest_container = self._setup_synced_containers(
+            dest_overrides=dest_account
+        )
+
+        # Create source and dest containers for segments in separate accounts.
+        # These containers must have same name for the destination SLO manifest
+        # to be able to resolve segments. Initially the destination has no sync
+        # key so segments will not sync.
+        segs_container = 'segments-%s' % uuid.uuid4()
+        dest_segs_info = dict(dest_account)
+        dest_segs_info.update({'name': segs_container, 'sync_key': None})
+        self._setup_synced_containers(
+            source_overrides={'name': segs_container, 'sync_key': 'segs_key'},
+            dest_overrides=dest_segs_info)
+
+        # upload a segment to source
+        segment_name = 'segment-%s' % uuid.uuid4()
+        segment_data = 'segment body'  # it's ok for first segment to be small
+        segment_etag = client.put_object(
+            self.url, self.token, segs_container, segment_name,
+            segment_data)
+
+        manifest = [{'etag': segment_etag,
+                     'size_bytes': len(segment_data),
+                     'path': '/%s/%s' % (segs_container, segment_name)}]
+        manifest_name = 'manifest-%s' % uuid.uuid4()
+        put_headers = {'X-Object-Meta-Test': 'put_value'}
+        client.put_object(
+            self.url, self.token, source_container, manifest_name,
+            json.dumps(manifest), headers=put_headers,
+            query_string='multipart-manifest=put')
+
+        resp_headers, manifest_body = client.get_object(
+            self.url, self.token, source_container, manifest_name,
+            query_string='multipart-manifest=get')
+        int_manifest = json.loads(manifest_body)
+
+        # cycle container-sync
+        Manager(['container-sync']).once()
+
+        # verify manifest was sync'd
+        resp_headers, dest_listing = client.get_container(
+            dest_account['url'], dest_account['token'], dest_container)
+        self.assertFalse(dest_listing[1:])
+        self.assertEqual(manifest_name, dest_listing[0]['name'])
+
+        # verify manifest body
+        resp_headers, body = client.get_object(
+            dest_account['url'], dest_account['token'], dest_container,
+            manifest_name, query_string='multipart-manifest=get')
+        self.assertEqual(int_manifest, json.loads(body))
+        self.assertIn('x-object-meta-test', resp_headers)
+        self.assertEqual('put_value', resp_headers['x-object-meta-test'])
+
+        # attempt to GET the SLO will fail because the segment wasn't sync'd
+        with self.assertRaises(ClientException) as cm:
+            client.get_object(dest_account['url'], dest_account['token'],
+                              dest_container, manifest_name)
+        self.assertEqual(409, cm.exception.http_status)
+
+        # now set sync key on destination segments container
+        client.put_container(
+            dest_account['url'], dest_account['token'], segs_container,
+            headers={'X-Container-Sync-Key': 'segs_key'})
+
+        # cycle container-sync
+        Manager(['container-sync']).once()
+
+        # sanity check - verify manifest body
+        resp_headers, body = client.get_object(
+            dest_account['url'], dest_account['token'], dest_container,
+            manifest_name, query_string='multipart-manifest=get')
+        self.assertEqual(int_manifest, json.loads(body))
+        self.assertIn('x-object-meta-test', resp_headers)
+        self.assertEqual('put_value', resp_headers['x-object-meta-test'])
+
+        # verify GET of SLO manifest now succeeds
+        resp_headers, body = client.get_object(
+            dest_account['url'], dest_account['token'], dest_container,
+            manifest_name)
+        self.assertEqual(segment_data, body)
+
     def test_sync_lazy_skey(self):
         # Create synced containers, but with no key at source
         source_container, dest_container =\
-            self._setup_synced_containers(None, 'secret')
+            self._setup_synced_containers(source_overrides={'sync_key': None})
 
         # upload to source
         object_name = 'object-%s' % uuid.uuid4()
@@ -178,7 +277,7 @@ class TestContainerSync(ReplProbeTest):
     def test_sync_lazy_dkey(self):
         # Create synced containers, but with no key at dest
         source_container, dest_container =\
-            self._setup_synced_containers('secret', None)
+            self._setup_synced_containers(dest_overrides={'sync_key': None})
 
         # upload to source
         object_name = 'object-%s' % uuid.uuid4()
