@@ -97,7 +97,7 @@ class TestObjectExpirer(ReplProbeTest):
             self.account, self.container_name, self.object_name,
             acceptable_statuses=(4,),
             headers={'X-Backend-Storage-Policy-Index': int(old_policy)})
-        self.assertTrue('x-backend-timestamp' in metadata)
+        self.assertIn('x-backend-timestamp', metadata)
         self.assertEqual(Timestamp(metadata['x-backend-timestamp']),
                          create_timestamp)
 
@@ -126,9 +126,9 @@ class TestObjectExpirer(ReplProbeTest):
                     self.fail('found object in %s and also %s' %
                               (found_in_policy, policy))
                 found_in_policy = policy
-                self.assertTrue('x-backend-timestamp' in metadata)
-                self.assertTrue(Timestamp(metadata['x-backend-timestamp']) >
-                                create_timestamp)
+                self.assertIn('x-backend-timestamp', metadata)
+                self.assertGreater(Timestamp(metadata['x-backend-timestamp']),
+                                   create_timestamp)
 
     def test_expirer_object_should_not_be_expired(self):
 
