@@ -34,7 +34,8 @@ from swift.obj.diskfile import write_metadata
 
 @patch_policies([StoragePolicy(0, 'zero', True),
                  StoragePolicy(1, 'one', False),
-                 StoragePolicy(2, 'two', False)])
+                 StoragePolicy(2, 'two', False),
+                 StoragePolicy(3, 'three', False)])
 class TestCliInfoBase(unittest.TestCase):
     def setUp(self):
         self.orig_hp = utils.HASH_PATH_PREFIX, utils.HASH_PATH_SUFFIX
@@ -72,6 +73,13 @@ class TestCliInfoBase(unittest.TestCase):
         # ... and another for policy 2
         self.two_ring_path = os.path.join(self.testdir, 'object-2.ring.gz')
         write_fake_ring(self.two_ring_path, *object_devs)
+        # ... and one for policy 3 with some v6 IPs in it
+        object_devs_ipv6 = [
+            {'ip': 'feed:face::dead:beef', 'port': 42},
+            {'ip': 'deca:fc0f:feeb:ad11::1', 'port': 43}
+        ]
+        self.three_ring_path = os.path.join(self.testdir, 'object-3.ring.gz')
+        write_fake_ring(self.three_ring_path, *object_devs_ipv6)
 
     def tearDown(self):
         utils.HASH_PATH_PREFIX, utils.HASH_PATH_SUFFIX = self.orig_hp
@@ -81,10 +89,9 @@ class TestCliInfoBase(unittest.TestCase):
         try:
             func(*args, **kwargs)
         except Exception as e:
-            self.assertTrue(msg in str(e),
-                            "Expected %r in %r" % (msg, str(e)))
-            self.assertTrue(isinstance(e, exc),
-                            "Expected %s, got %s" % (exc, type(e)))
+            self.assertIn(msg, str(e), "Expected %r in %r" % (msg, str(e)))
+            self.assertIsInstance(e, exc,
+                                  "Expected %s, got %s" % (exc, type(e)))
 
 
 class TestCliInfo(TestCliInfoBase):
@@ -198,9 +205,9 @@ No user metadata found in db file''' % POLICIES[0].name
             print_ring_locations(acctring, 'dir', 'acct')
         exp_db = os.path.join('${DEVICE:-/srv/node*}', 'sdb1', 'dir', '3',
                               'b47', 'dc5be2aa4347a22a0fee6bc7de505b47')
-        self.assertTrue(exp_db in out.getvalue())
-        self.assertTrue('127.0.0.1' in out.getvalue())
-        self.assertTrue('127.0.0.2' in out.getvalue())
+        self.assertIn(exp_db, out.getvalue())
+        self.assertIn('127.0.0.1', out.getvalue())
+        self.assertIn('127.0.0.2', out.getvalue())
 
     def test_print_ring_locations_container(self):
         out = StringIO()
@@ -209,7 +216,7 @@ No user metadata found in db file''' % POLICIES[0].name
             print_ring_locations(contring, 'dir', 'acct', 'con')
         exp_db = os.path.join('${DEVICE:-/srv/node*}', 'sdb1', 'dir', '1',
                               'fe6', '63e70955d78dfc62821edc07d6ec1fe6')
-        self.assertTrue(exp_db in out.getvalue())
+        self.assertIn(exp_db, out.getvalue())
 
     def test_print_ring_locations_obj(self):
         out = StringIO()
@@ -218,7 +225,7 @@ No user metadata found in db file''' % POLICIES[0].name
             print_ring_locations(objring, 'dir', 'acct', 'con', 'obj')
         exp_obj = os.path.join('${DEVICE:-/srv/node*}', 'sda1', 'dir', '1',
                                '117', '4a16154fc15c75e26ba6afadf5b1c117')
-        self.assertTrue(exp_obj in out.getvalue())
+        self.assertIn(exp_obj, out.getvalue())
 
     def test_print_ring_locations_partition_number(self):
         out = StringIO()
@@ -229,8 +236,8 @@ No user metadata found in db file''' % POLICIES[0].name
                                 'objects', '1')
         exp_obj2 = os.path.join('${DEVICE:-/srv/node*}', 'sdb1',
                                 'objects', '1')
-        self.assertTrue(exp_obj1 in out.getvalue())
-        self.assertTrue(exp_obj2 in out.getvalue())
+        self.assertIn(exp_obj1, out.getvalue())
+        self.assertIn(exp_obj2, out.getvalue())
 
     def test_print_item_locations_invalid_args(self):
         # No target specified
@@ -257,9 +264,9 @@ No user metadata found in db file''' % POLICIES[0].name
             # Test mismatch of ring and policy name (valid policy)
             self.assertRaises(InfoSystemExit, print_item_locations,
                               objring, policy_name='zero')
-        self.assertTrue('Warning: mismatch between ring and policy name!'
-                        in out.getvalue())
-        self.assertTrue('No target specified' in out.getvalue())
+        self.assertIn('Warning: mismatch between ring and policy name!',
+                      out.getvalue())
+        self.assertIn('No target specified', out.getvalue())
 
     def test_print_item_locations_invalid_policy_no_target(self):
         out = StringIO()
@@ -269,8 +276,8 @@ No user metadata found in db file''' % POLICIES[0].name
             self.assertRaises(InfoSystemExit, print_item_locations,
                               objring, policy_name=policy_name)
         exp_msg = 'Warning: Policy %s is not valid' % policy_name
-        self.assertTrue(exp_msg in out.getvalue())
-        self.assertTrue('No target specified' in out.getvalue())
+        self.assertIn(exp_msg, out.getvalue())
+        self.assertIn('No target specified', out.getvalue())
 
     def test_print_item_locations_policy_object(self):
         out = StringIO()
@@ -282,10 +289,10 @@ No user metadata found in db file''' % POLICIES[0].name
         exp_acct_msg = 'Account  \tNone'
         exp_cont_msg = 'Container\tNone'
         exp_obj_msg = 'Object   \tNone'
-        self.assertTrue(exp_part_msg in out.getvalue())
-        self.assertTrue(exp_acct_msg in out.getvalue())
-        self.assertTrue(exp_cont_msg in out.getvalue())
-        self.assertTrue(exp_obj_msg in out.getvalue())
+        self.assertIn(exp_part_msg, out.getvalue())
+        self.assertIn(exp_acct_msg, out.getvalue())
+        self.assertIn(exp_cont_msg, out.getvalue())
+        self.assertIn(exp_obj_msg, out.getvalue())
 
     def test_print_item_locations_dashed_ring_name_partition(self):
         out = StringIO()
@@ -298,10 +305,10 @@ No user metadata found in db file''' % POLICIES[0].name
         exp_acct_msg = 'Account  \tNone'
         exp_cont_msg = 'Container\tNone'
         exp_obj_msg = 'Object   \tNone'
-        self.assertTrue(exp_part_msg in out.getvalue())
-        self.assertTrue(exp_acct_msg in out.getvalue())
-        self.assertTrue(exp_cont_msg in out.getvalue())
-        self.assertTrue(exp_obj_msg in out.getvalue())
+        self.assertIn(exp_part_msg, out.getvalue())
+        self.assertIn(exp_acct_msg, out.getvalue())
+        self.assertIn(exp_cont_msg, out.getvalue())
+        self.assertIn(exp_obj_msg, out.getvalue())
 
     def test_print_item_locations_account_with_ring(self):
         out = StringIO()
@@ -310,16 +317,16 @@ No user metadata found in db file''' % POLICIES[0].name
             account_ring = ring.Ring(self.testdir, ring_name=account)
             print_item_locations(account_ring, account=account)
         exp_msg = 'Account  \t%s' % account
-        self.assertTrue(exp_msg in out.getvalue())
+        self.assertIn(exp_msg, out.getvalue())
         exp_warning = 'Warning: account specified ' + \
                       'but ring not named "account"'
-        self.assertTrue(exp_warning in out.getvalue())
+        self.assertIn(exp_warning, out.getvalue())
         exp_acct_msg = 'Account  \t%s' % account
         exp_cont_msg = 'Container\tNone'
         exp_obj_msg = 'Object   \tNone'
-        self.assertTrue(exp_acct_msg in out.getvalue())
-        self.assertTrue(exp_cont_msg in out.getvalue())
-        self.assertTrue(exp_obj_msg in out.getvalue())
+        self.assertIn(exp_acct_msg, out.getvalue())
+        self.assertIn(exp_cont_msg, out.getvalue())
+        self.assertIn(exp_obj_msg, out.getvalue())
 
     def test_print_item_locations_account_no_ring(self):
         out = StringIO()
@@ -330,9 +337,9 @@ No user metadata found in db file''' % POLICIES[0].name
         exp_acct_msg = 'Account  \t%s' % account
         exp_cont_msg = 'Container\tNone'
         exp_obj_msg = 'Object   \tNone'
-        self.assertTrue(exp_acct_msg in out.getvalue())
-        self.assertTrue(exp_cont_msg in out.getvalue())
-        self.assertTrue(exp_obj_msg in out.getvalue())
+        self.assertIn(exp_acct_msg, out.getvalue())
+        self.assertIn(exp_cont_msg, out.getvalue())
+        self.assertIn(exp_obj_msg, out.getvalue())
 
     def test_print_item_locations_account_container_ring(self):
         out = StringIO()
@@ -345,9 +352,9 @@ No user metadata found in db file''' % POLICIES[0].name
         exp_acct_msg = 'Account  \t%s' % account
         exp_cont_msg = 'Container\t%s' % container
         exp_obj_msg = 'Object   \tNone'
-        self.assertTrue(exp_acct_msg in out.getvalue())
-        self.assertTrue(exp_cont_msg in out.getvalue())
-        self.assertTrue(exp_obj_msg in out.getvalue())
+        self.assertIn(exp_acct_msg, out.getvalue())
+        self.assertIn(exp_cont_msg, out.getvalue())
+        self.assertIn(exp_obj_msg, out.getvalue())
 
     def test_print_item_locations_account_container_no_ring(self):
         out = StringIO()
@@ -359,9 +366,9 @@ No user metadata found in db file''' % POLICIES[0].name
         exp_acct_msg = 'Account  \t%s' % account
         exp_cont_msg = 'Container\t%s' % container
         exp_obj_msg = 'Object   \tNone'
-        self.assertTrue(exp_acct_msg in out.getvalue())
-        self.assertTrue(exp_cont_msg in out.getvalue())
-        self.assertTrue(exp_obj_msg in out.getvalue())
+        self.assertIn(exp_acct_msg, out.getvalue())
+        self.assertIn(exp_cont_msg, out.getvalue())
+        self.assertIn(exp_obj_msg, out.getvalue())
 
     def test_print_item_locations_account_container_object_ring(self):
         out = StringIO()
@@ -376,9 +383,9 @@ No user metadata found in db file''' % POLICIES[0].name
         exp_acct_msg = 'Account  \t%s' % account
         exp_cont_msg = 'Container\t%s' % container
         exp_obj_msg = 'Object   \t%s' % obj
-        self.assertTrue(exp_acct_msg in out.getvalue())
-        self.assertTrue(exp_cont_msg in out.getvalue())
-        self.assertTrue(exp_obj_msg in out.getvalue())
+        self.assertIn(exp_acct_msg, out.getvalue())
+        self.assertIn(exp_cont_msg, out.getvalue())
+        self.assertIn(exp_obj_msg, out.getvalue())
 
     def test_print_item_locations_account_container_object_dashed_ring(self):
         out = StringIO()
@@ -393,9 +400,9 @@ No user metadata found in db file''' % POLICIES[0].name
         exp_acct_msg = 'Account  \t%s' % account
         exp_cont_msg = 'Container\t%s' % container
         exp_obj_msg = 'Object   \t%s' % obj
-        self.assertTrue(exp_acct_msg in out.getvalue())
-        self.assertTrue(exp_cont_msg in out.getvalue())
-        self.assertTrue(exp_obj_msg in out.getvalue())
+        self.assertIn(exp_acct_msg, out.getvalue())
+        self.assertIn(exp_cont_msg, out.getvalue())
+        self.assertIn(exp_obj_msg, out.getvalue())
 
     def test_print_info(self):
         db_file = 'foo'
@@ -423,7 +430,7 @@ No user metadata found in db file''' % POLICIES[0].name
         if exp_raised:
             self.fail("Unexpected exception raised")
         else:
-            self.assertTrue(len(out.getvalue().strip()) > 800)
+            self.assertGreater(len(out.getvalue().strip()), 800)
 
         controller = ContainerController(
             {'devices': self.testdir, 'mount_check': 'false'})
@@ -451,7 +458,7 @@ No user metadata found in db file''' % POLICIES[0].name
         if exp_raised:
             self.fail("Unexpected exception raised")
         else:
-            self.assertTrue(len(out.getvalue().strip()) > 600)
+            self.assertGreater(len(out.getvalue().strip()), 600)
 
         out = StringIO()
         exp_raised = False
@@ -509,8 +516,8 @@ class TestPrintObj(TestCliInfoBase):
             print_obj(self.datafile, swift_dir=self.testdir)
         etag_msg = 'ETag: Not found in metadata'
         length_msg = 'Content-Length: Not found in metadata'
-        self.assertTrue(etag_msg in out.getvalue())
-        self.assertTrue(length_msg in out.getvalue())
+        self.assertIn(etag_msg, out.getvalue())
+        self.assertIn(length_msg, out.getvalue())
 
     def test_print_obj_with_policy(self):
         out = StringIO()
@@ -519,15 +526,15 @@ class TestPrintObj(TestCliInfoBase):
         etag_msg = 'ETag: Not found in metadata'
         length_msg = 'Content-Length: Not found in metadata'
         ring_loc_msg = 'ls -lah'
-        self.assertTrue(etag_msg in out.getvalue())
-        self.assertTrue(length_msg in out.getvalue())
-        self.assertTrue(ring_loc_msg in out.getvalue())
+        self.assertIn(etag_msg, out.getvalue())
+        self.assertIn(length_msg, out.getvalue())
+        self.assertIn(ring_loc_msg, out.getvalue())
 
     def test_missing_etag(self):
         out = StringIO()
         with mock.patch('sys.stdout', out):
             print_obj(self.datafile)
-        self.assertTrue('ETag: Not found in metadata' in out.getvalue())
+        self.assertIn('ETag: Not found in metadata', out.getvalue())
 
 
 class TestPrintObjFullMeta(TestCliInfoBase):
@@ -550,7 +557,7 @@ class TestPrintObjFullMeta(TestCliInfoBase):
         out = StringIO()
         with mock.patch('sys.stdout', out):
             print_obj(self.datafile, swift_dir=self.testdir)
-        self.assertTrue('/objects-1/' in out.getvalue())
+        self.assertIn('/objects-1/', out.getvalue())
 
     def test_print_obj_policy_index(self):
         # Check an output of policy index when current directory is in
@@ -567,7 +574,89 @@ class TestPrintObjFullMeta(TestCliInfoBase):
                 print_obj(file_name, swift_dir=self.testdir)
         finally:
             os.chdir(cwd)
-        self.assertTrue('X-Backend-Storage-Policy-Index: 1' in out.getvalue())
+        self.assertIn('X-Backend-Storage-Policy-Index: 1', out.getvalue())
+
+    def test_print_obj_curl_command_ipv4(self):
+        # Note: policy 2 has IPv4 addresses in its ring
+        datafile2 = os.path.join(
+            self.testdir,
+            'sda', 'objects-2', '1', 'ea8',
+            'db4449e025aca992307c7c804a67eea8', '1402017884.18202.data')
+        utils.mkdirs(os.path.dirname(datafile2))
+        with open(datafile2, 'wb') as fp:
+            md = {'name': '/AUTH_admin/c/obj',
+                  'Content-Type': 'application/octet-stream',
+                  'ETag': 'd41d8cd98f00b204e9800998ecf8427e',
+                  'Content-Length': 0}
+            write_metadata(fp, md)
+
+        object_ring = ring.Ring(self.testdir, ring_name='object-2')
+        part, nodes = object_ring.get_nodes('AUTH_admin', 'c', 'obj')
+        node = nodes[0]
+
+        out = StringIO()
+        hash_dir = os.path.dirname(datafile2)
+        file_name = os.path.basename(datafile2)
+
+        # Change working directory to object hash dir
+        cwd = os.getcwd()
+        try:
+            os.chdir(hash_dir)
+            with mock.patch('sys.stdout', out):
+                print_obj(file_name, swift_dir=self.testdir)
+        finally:
+            os.chdir(cwd)
+
+        exp_curl = (
+            'curl -g -I -XHEAD '
+            '"http://{host}:{port}/{device}/{part}/AUTH_admin/c/obj" '
+            '-H "X-Backend-Storage-Policy-Index: 2"').format(
+                host=node['ip'],
+                port=node['port'],
+                device=node['device'],
+                part=part)
+        self.assertIn(exp_curl, out.getvalue())
+
+    def test_print_obj_curl_command_ipv6(self):
+        # Note: policy 3 has IPv6 addresses in its ring
+        datafile3 = os.path.join(
+            self.testdir,
+            'sda', 'objects-3', '1', 'ea8',
+            'db4449e025aca992307c7c804a67eea8', '1402017884.18202.data')
+        utils.mkdirs(os.path.dirname(datafile3))
+        with open(datafile3, 'wb') as fp:
+            md = {'name': '/AUTH_admin/c/obj',
+                  'Content-Type': 'application/octet-stream',
+                  'ETag': 'd41d8cd98f00b204e9800998ecf8427e',
+                  'Content-Length': 0}
+            write_metadata(fp, md)
+
+        object_ring = ring.Ring(self.testdir, ring_name='object-3')
+        part, nodes = object_ring.get_nodes('AUTH_admin', 'c', 'obj')
+        node = nodes[0]
+
+        out = StringIO()
+        hash_dir = os.path.dirname(datafile3)
+        file_name = os.path.basename(datafile3)
+
+        # Change working directory to object hash dir
+        cwd = os.getcwd()
+        try:
+            os.chdir(hash_dir)
+            with mock.patch('sys.stdout', out):
+                print_obj(file_name, swift_dir=self.testdir)
+        finally:
+            os.chdir(cwd)
+
+        exp_curl = (
+            'curl -g -I -XHEAD '
+            '"http://[{host}]:{port}'
+            '/{device}/{part}/AUTH_admin/c/obj" ').format(
+                host=node['ip'],
+                port=node['port'],
+                device=node['device'],
+                part=part)
+        self.assertIn(exp_curl, out.getvalue())
 
     def test_print_obj_meta_and_ts_files(self):
         # verify that print_obj will also read from meta and ts files
@@ -578,7 +667,7 @@ class TestPrintObjFullMeta(TestCliInfoBase):
             out = StringIO()
             with mock.patch('sys.stdout', out):
                 print_obj(test_file, swift_dir=self.testdir)
-            self.assertTrue('/objects-1/' in out.getvalue())
+            self.assertIn('/objects-1/', out.getvalue())
 
     def test_print_obj_no_ring(self):
         no_rings_dir = os.path.join(self.testdir, 'no_rings_here')
@@ -587,22 +676,22 @@ class TestPrintObjFullMeta(TestCliInfoBase):
         out = StringIO()
         with mock.patch('sys.stdout', out):
             print_obj(self.datafile, swift_dir=no_rings_dir)
-        self.assertTrue('d41d8cd98f00b204e9800998ecf8427e' in out.getvalue())
-        self.assertTrue('Partition' not in out.getvalue())
+        self.assertIn('d41d8cd98f00b204e9800998ecf8427e', out.getvalue())
+        self.assertNotIn('Partition', out.getvalue())
 
     def test_print_obj_policy_name_mismatch(self):
         out = StringIO()
         with mock.patch('sys.stdout', out):
             print_obj(self.datafile, policy_name='two', swift_dir=self.testdir)
         ring_alert_msg = 'Warning: Ring does not match policy!'
-        self.assertTrue(ring_alert_msg in out.getvalue())
+        self.assertIn(ring_alert_msg, out.getvalue())
 
     def test_valid_etag(self):
         out = StringIO()
         with mock.patch('sys.stdout', out):
             print_obj(self.datafile)
-        self.assertTrue('ETag: d41d8cd98f00b204e9800998ecf8427e (valid)'
-                        in out.getvalue())
+        self.assertIn('ETag: d41d8cd98f00b204e9800998ecf8427e (valid)',
+                      out.getvalue())
 
     def test_invalid_etag(self):
         with open(self.datafile, 'wb') as fp:
@@ -615,24 +704,26 @@ class TestPrintObjFullMeta(TestCliInfoBase):
         out = StringIO()
         with mock.patch('sys.stdout', out):
             print_obj(self.datafile)
-        self.assertTrue('ETag: badetag doesn\'t match file hash'
-                        in out.getvalue())
+        self.assertIn('ETag: badetag doesn\'t match file hash',
+                      out.getvalue())
 
     def test_unchecked_etag(self):
         out = StringIO()
         with mock.patch('sys.stdout', out):
             print_obj(self.datafile, check_etag=False)
-        self.assertTrue('ETag: d41d8cd98f00b204e9800998ecf8427e (not checked)'
-                        in out.getvalue())
+        self.assertIn('ETag: d41d8cd98f00b204e9800998ecf8427e (not checked)',
+                      out.getvalue())
 
     def test_print_obj_metadata(self):
         self.assertRaisesMessage(ValueError, 'Metadata is None',
                                  print_obj_metadata, [])
 
         def get_metadata(items):
-            md = dict(name='/AUTH_admin/c/dummy')
-            md['Content-Type'] = 'application/octet-stream'
-            md['X-Timestamp'] = 106.3
+            md = {
+                'name': '/AUTH_admin/c/dummy',
+                'Content-Type': 'application/octet-stream',
+                'X-Timestamp': 106.3,
+            }
             md.update(items)
             return md
 
@@ -649,6 +740,8 @@ Content-Type: application/octet-stream
 Timestamp: 1970-01-01T00:01:46.300000 (%s)
 System Metadata:
   No metadata found
+Transient System Metadata:
+  No metadata found
 User Metadata:
   X-Object-Meta-Mtime: 107.3
 Other Metadata:
@@ -674,6 +767,8 @@ Timestamp: 1970-01-01T00:01:46.300000 (%s)
 System Metadata:
   X-Object-Sysmeta-Mtime: 107.3
   X-Object-Sysmeta-Name: Obj name
+Transient System Metadata:
+  No metadata found
 User Metadata:
   No metadata found
 Other Metadata:
@@ -699,6 +794,8 @@ Content-Type: application/octet-stream
 Timestamp: 1970-01-01T00:01:46.300000 (%s)
 System Metadata:
   X-Object-Sysmeta-Mtime: 107.3
+Transient System Metadata:
+  No metadata found
 User Metadata:
   X-Object-Meta-Mtime: 107.3
 Other Metadata:
@@ -720,6 +817,8 @@ Content-Type: application/octet-stream
 Timestamp: 1970-01-01T00:01:46.300000 (%s)
 System Metadata:
   No metadata found
+Transient System Metadata:
+  No metadata found
 User Metadata:
   No metadata found
 Other Metadata:
@@ -743,6 +842,8 @@ Content-Type: application/octet-stream
 Timestamp: 1970-01-01T00:01:46.300000 (%s)
 System Metadata:
   No metadata found
+Transient System Metadata:
+  No metadata found
 User Metadata:
   X-Object-Meta-Mtime: 107.3
 Other Metadata:
@@ -765,6 +866,8 @@ Content-Type: Not found in metadata
 Timestamp: 1970-01-01T00:01:46.300000 (%s)
 System Metadata:
   No metadata found
+Transient System Metadata:
+  No metadata found
 User Metadata:
   X-Object-Meta-Mtime: 107.3
 Other Metadata:
@@ -787,6 +890,8 @@ Content-Type: application/octet-stream
 Timestamp: Not found in metadata
 System Metadata:
   No metadata found
+Transient System Metadata:
+  No metadata found
 User Metadata:
   X-Object-Meta-Mtime: 107.3
 Other Metadata:
@@ -794,6 +899,34 @@ Other Metadata:
 
         self.assertEqual(out.getvalue().strip(), exp_out)
 
+        metadata = get_metadata({
+            'X-Object-Meta-Mtime': '107.3',
+            'X-Object-Sysmeta-Mtime': '106.3',
+            'X-Object-Transient-Sysmeta-Mtime': '105.3',
+            'X-Object-Mtime': '104.3',
+        })
+        out = StringIO()
+        with mock.patch('sys.stdout', out):
+            print_obj_metadata(metadata)
+        exp_out = '''Path: /AUTH_admin/c/dummy
+  Account: AUTH_admin
+  Container: c
+  Object: dummy
+  Object hash: 128fdf98bddd1b1e8695f4340e67a67a
+Content-Type: application/octet-stream
+Timestamp: 1970-01-01T00:01:46.300000 (%s)
+System Metadata:
+  X-Object-Sysmeta-Mtime: 106.3
+Transient System Metadata:
+  X-Object-Transient-Sysmeta-Mtime: 105.3
+User Metadata:
+  X-Object-Meta-Mtime: 107.3
+Other Metadata:
+  X-Object-Mtime: 104.3''' % (
+            utils.Timestamp(106.3).internal)
+
+        self.assertEqual(out.getvalue().strip(), exp_out)
+
 
 class TestPrintObjWeirdPath(TestPrintObjFullMeta):
     def setUp(self):
