@@ -27,6 +27,7 @@ from swift.common.swob import Request, Response
 from swift.common import constraints
 from swift.common.storage_policy import StoragePolicy
 from test.unit import patch_policies
+from test.unit.common.middleware.helpers import FakeAppThatExcepts
 
 
 class FakeApp(object):
@@ -59,12 +60,6 @@ class FakeApp(object):
         return self.body
 
 
-class FakeAppThatExcepts(object):
-
-    def __call__(self, env, start_response):
-        raise Exception("We take exception to that!")
-
-
 class FakeAppNoContentLengthNoTransferEncoding(object):
 
     def __init__(self, body=None):
@@ -460,8 +455,8 @@ class TestProxyLogging(unittest.TestCase):
         headers = unquote(log_parts[14]).split('\n')
         self.assertTrue('First: 1' in headers)
         self.assertTrue('Second: 2' in headers)
-        self.assertTrue('Third: 3' not in headers)
-        self.assertTrue('Host: localhost:80' not in headers)
+        self.assertNotIn('Third: 3', headers)
+        self.assertNotIn('Host: localhost:80', headers)
 
     def test_upload_size(self):
         # Using default policy
