@@ -21,12 +21,14 @@ import logging
 import errno
 import math
 import time
-from mock import patch, call
 from shutil import rmtree, copy
 from tempfile import mkdtemp, NamedTemporaryFile
-import mock
 import json
 
+import mock
+from mock import patch, call
+from six.moves import reload_module
+
 from swift.container.backend import DATADIR
 from swift.common import db_replicator
 from swift.common.utils import (normalize_timestamp, hash_path,
@@ -36,6 +38,7 @@ from swift.common.swob import HTTPException
 
 from test import unit
 from test.unit.common.test_db import ExampleBroker
+from test.unit import with_tempdir
 
 
 TEST_ACCOUNT_NAME = 'a c t'
@@ -44,7 +47,7 @@ TEST_CONTAINER_NAME = 'c o n'
 
 def teardown_module():
     "clean up my monkey patching"
-    reload(db_replicator)
+    reload_module(db_replicator)
 
 
 @contextmanager
@@ -72,8 +75,8 @@ class FakeRing(object):
 class FakeRingWithSingleNode(object):
     class Ring(object):
         devs = [dict(
-            id=1, weight=10.0, zone=1, ip='1.1.1.1', port=6000, device='sdb',
-            meta='', replication_ip='1.1.1.1', replication_port=6000
+            id=1, weight=10.0, zone=1, ip='1.1.1.1', port=6200, device='sdb',
+            meta='', replication_ip='1.1.1.1', replication_port=6200, region=1
         )]
 
         def __init__(self, path, reload_time=15, ring_name=None):
@@ -92,23 +95,23 @@ class FakeRingWithSingleNode(object):
 class FakeRingWithNodes(object):
     class Ring(object):
         devs = [dict(
-            id=1, weight=10.0, zone=1, ip='1.1.1.1', port=6000, device='sdb',
-            meta='', replication_ip='1.1.1.1', replication_port=6000, region=1
+            id=1, weight=10.0, zone=1, ip='1.1.1.1', port=6200, device='sdb',
+            meta='', replication_ip='1.1.1.1', replication_port=6200, region=1
         ), dict(
-            id=2, weight=10.0, zone=2, ip='1.1.1.2', port=6000, device='sdb',
-            meta='', replication_ip='1.1.1.2', replication_port=6000, region=2
+            id=2, weight=10.0, zone=2, ip='1.1.1.2', port=6200, device='sdb',
+            meta='', replication_ip='1.1.1.2', replication_port=6200, region=2
         ), dict(
-            id=3, weight=10.0, zone=3, ip='1.1.1.3', port=6000, device='sdb',
-            meta='', replication_ip='1.1.1.3', replication_port=6000, region=1
+            id=3, weight=10.0, zone=3, ip='1.1.1.3', port=6200, device='sdb',
+            meta='', replication_ip='1.1.1.3', replication_port=6200, region=1
         ), dict(
-            id=4, weight=10.0, zone=4, ip='1.1.1.4', port=6000, device='sdb',
-            meta='', replication_ip='1.1.1.4', replication_port=6000, region=2
+            id=4, weight=10.0, zone=4, ip='1.1.1.4', port=6200, device='sdb',
+            meta='', replication_ip='1.1.1.4', replication_port=6200, region=2
         ), dict(
-            id=5, weight=10.0, zone=5, ip='1.1.1.5', port=6000, device='sdb',
-            meta='', replication_ip='1.1.1.5', replication_port=6000, region=1
+            id=5, weight=10.0, zone=5, ip='1.1.1.5', port=6200, device='sdb',
+            meta='', replication_ip='1.1.1.5', replication_port=6200, region=1
         ), dict(
-            id=6, weight=10.0, zone=6, ip='1.1.1.6', port=6000, device='sdb',
-            meta='', replication_ip='1.1.1.6', replication_port=6000, region=2
+            id=6, weight=10.0, zone=6, ip='1.1.1.6', port=6200, device='sdb',
+            meta='', replication_ip='1.1.1.6', replication_port=6200, region=2
         )]
 
         def __init__(self, path, reload_time=15, ring_name=None):
@@ -486,7 +489,7 @@ class TestDBReplicator(unittest.TestCase):
         base = 'swift.common.db_replicator.'
         with patch(base + 'whataremyips', return_value=['1.1.1.1']), \
                 patch(base + 'ring', FakeRingWithNodes()):
-            replicator = TestReplicator({'bind_port': 6000,
+            replicator = TestReplicator({'bind_port': 6200,
                                          'recon_cache_path': self.recon_cache},
                                         logger=logger)
             replicator.run_once()
@@ -507,10 +510,11 @@ class TestDBReplicator(unittest.TestCase):
         db_replicator.ring = FakeRingWithSingleNode()
         # If a bind_ip is specified, it's plumbed into whataremyips() and
         # returned by itself.
-        conf = {'mount_check': 'true', 'bind_ip': '1.1.1.1', 'bind_port': 6000}
+        conf = {'mount_check': 'true', 'bind_ip': '1.1.1.1',
+                'bind_port': 6200}
         replicator = TestReplicator(conf, logger=unit.FakeLogger())
         self.assertEqual(replicator.mount_check, True)
-        self.assertEqual(replicator.port, 6000)
+        self.assertEqual(replicator.port, 6200)
 
         def mock_ismount(path):
             self.assertEqual(path,
@@ -528,10 +532,10 @@ class TestDBReplicator(unittest.TestCase):
 
     def test_run_once_node_is_mounted(self):
         db_replicator.ring = FakeRingWithSingleNode()
-        conf = {'mount_check': 'true', 'bind_port': 6000}
+        conf = {'mount_check': 'true', 'bind_port': 6200}
         replicator = TestReplicator(conf, logger=unit.FakeLogger())
         self.assertEqual(replicator.mount_check, True)
-        self.assertEqual(replicator.port, 6000)
+        self.assertEqual(replicator.port, 6200)
 
         def mock_unlink_older_than(path, mtime):
             self.assertEqual(path,
@@ -632,6 +636,9 @@ class TestDBReplicator(unittest.TestCase):
 
     def test_replicate_object_delete_because_not_shouldbehere(self):
         replicator = TestReplicator({})
+        replicator.ring = FakeRingWithNodes().Ring('path')
+        replicator.brokerclass = FakeAccountBroker
+        replicator._repl_to_node = lambda *args: True
         replicator.delete_db = self.stub_delete_db
         replicator._replicate_object('0', '/path/to/file', 'node_id')
         self.assertEqual(['/path/to/file'], self.delete_db_calls)
@@ -668,6 +675,30 @@ class TestDBReplicator(unittest.TestCase):
             [(('Found /path/to/file for /a%20c%20t/c%20o%20n when it should '
                'be on partition 0; will replicate out and remove.',), {})])
 
+    def test_replicate_container_out_of_place_no_node(self):
+        replicator = TestReplicator({}, logger=unit.FakeLogger())
+        replicator.ring = FakeRingWithSingleNode().Ring('path')
+        replicator._repl_to_node = lambda *args: True
+
+        replicator.delete_db = self.stub_delete_db
+        # Correct node_id, wrong part
+        part = replicator.ring.get_part(
+            TEST_ACCOUNT_NAME, TEST_CONTAINER_NAME) + 1
+        node_id = replicator.ring.get_part_nodes(part)[0]['id']
+        replicator._replicate_object(str(part), '/path/to/file', node_id)
+        self.assertEqual(['/path/to/file'], self.delete_db_calls)
+
+        self.delete_db_calls = []
+
+        # No nodes this time
+        replicator.ring.get_part_nodes = lambda *args: []
+        replicator.delete_db = self.stub_delete_db
+        # Correct node_id, wrong part
+        part = replicator.ring.get_part(
+            TEST_ACCOUNT_NAME, TEST_CONTAINER_NAME) + 1
+        replicator._replicate_object(str(part), '/path/to/file', node_id)
+        self.assertEqual([], self.delete_db_calls)
+
     def test_replicate_object_different_region(self):
         db_replicator.ring = FakeRingWithNodes()
         replicator = TestReplicator({})
@@ -1020,6 +1051,150 @@ class TestDBReplicator(unittest.TestCase):
         finally:
             rmtree(drive)
 
+    @with_tempdir
+    def test_empty_suffix_and_hash_dirs_get_cleanedup(self, tempdir):
+        # tests empty suffix and hash dirs of a quarantined db are cleaned up
+        listdir_calls = []
+        isdir_calls = []
+        exists_calls = []
+        shuffle_calls = []
+        rmdir_calls = []
+
+        existing_file = os.path.join(tempdir, 'a_file_exists')
+        open(existing_file, 'a').close()
+        dir_with_no_obj_file = '/srv/node/sdb/containers/9876/xyz/' \
+                               '11111111111111111111111111111xyz'
+
+        def _listdir(path):
+            listdir_calls.append(path)
+            if not path.startswith('/srv/node/sda/containers') and \
+                    not path.startswith('/srv/node/sdb/containers'):
+                return []
+            path = path[len('/srv/node/sdx/containers'):]
+            if path == '':
+                return ['6789', '9876']
+            elif path == '/9876':
+                return ['xyz']
+            elif path == '/9876/xyz':
+                return ['11111111111111111111111111111xyz']
+            elif path == '/9876/xyz/11111111111111111111111111111xyz':
+                return []
+            elif path == '/6789':
+                return ['jkl']
+            elif path == '/6789/jkl':
+                return []
+            return []
+
+        def _isdir(path):
+            isdir_calls.append(path)
+            if not path.startswith('/srv/node/sda/containers') and \
+                    not path.startswith('/srv/node/sdb/containers'):
+                return False
+            path = path[len('/srv/node/sdx/containers'):]
+            if path in ('/9876', '/9876/xyz',
+                        '/9876/xyz/11111111111111111111111111111xyz',
+                        '/6789', '/6789/jkl'):
+                return True
+            return False
+
+        def _exists(arg):
+            exists_calls.append(arg)
+            if arg in ('/srv/node/sda/containers/9876/xyz/'
+                       '11111111111111111111111111111xyz/'
+                       '11111111111111111111111111111xyz.db',
+                       '/srv/node/sdb/containers/9876/xyz/'
+                       '11111111111111111111111111111xyz/'
+                       '11111111111111111111111111111xyz.db'):
+                return False
+            return True
+
+        def _shuffle(arg):
+            shuffle_calls.append(arg)
+
+        def _rmdir(arg):
+            rmdir_calls.append(arg)
+            if arg == dir_with_no_obj_file:
+                # use db_replicator.os.rmdir to delete a directory with an
+                # existing file; fail if it doesn't handle OSError
+                orig_rmdir(tempdir)
+                self.fail('The rmdir did not handle the exception as expected')
+
+        orig_listdir = db_replicator.os.listdir
+        orig_isdir = db_replicator.os.path.isdir
+        orig_exists = db_replicator.os.path.exists
+        orig_shuffle = db_replicator.random.shuffle
+        orig_rmdir = db_replicator.os.rmdir
+
+        try:
+            db_replicator.os.listdir = _listdir
+            db_replicator.os.path.isdir = _isdir
+            db_replicator.os.path.exists = _exists
+            db_replicator.random.shuffle = _shuffle
+            db_replicator.os.rmdir = _rmdir
+
+            datadirs = [('/srv/node/sda/containers', 1),
+                        ('/srv/node/sdb/containers', 2)]
+            results = list(db_replicator.roundrobin_datadirs(datadirs))
+            # The results show that there are no .db files to be returned
+            # in this case
+            self.assertEqual(results, [])
+            # The listdir calls show that we only listdir the dirs
+            self.assertEqual(listdir_calls, [
+                '/srv/node/sda/containers',
+                '/srv/node/sda/containers/6789',
+                '/srv/node/sda/containers/6789/jkl',
+                '/srv/node/sda/containers/9876',
+                '/srv/node/sda/containers/9876/xyz',
+                '/srv/node/sdb/containers',
+                '/srv/node/sdb/containers/6789',
+                '/srv/node/sdb/containers/6789/jkl',
+                '/srv/node/sdb/containers/9876',
+                '/srv/node/sdb/containers/9876/xyz'])
+            # The isdir calls show that we did ask about the things pretending
+            # to be files at various levels.
+            self.assertEqual(isdir_calls, [
+                '/srv/node/sda/containers/6789',
+                '/srv/node/sda/containers/6789/jkl',
+                '/srv/node/sda/containers/9876',
+                '/srv/node/sda/containers/9876/xyz',
+                ('/srv/node/sda/containers/9876/xyz/'
+                 '11111111111111111111111111111xyz'),
+                '/srv/node/sdb/containers/6789',
+                '/srv/node/sdb/containers/6789/jkl',
+                '/srv/node/sdb/containers/9876',
+                '/srv/node/sdb/containers/9876/xyz',
+                ('/srv/node/sdb/containers/9876/xyz/'
+                 '11111111111111111111111111111xyz')])
+            # The exists calls are the .db files we looked for as we walked the
+            # structure.
+            self.assertEqual(exists_calls, [
+                ('/srv/node/sda/containers/9876/xyz/'
+                 '11111111111111111111111111111xyz/'
+                 '11111111111111111111111111111xyz.db'),
+                ('/srv/node/sdb/containers/9876/xyz/'
+                 '11111111111111111111111111111xyz/'
+                 '11111111111111111111111111111xyz.db')])
+            # Shows that we called shuffle twice, once for each device.
+            self.assertEqual(
+                shuffle_calls, [['6789', '9876'],
+                                ['6789', '9876']])
+
+            # Shows that we called rmdir and removed directories with no db
+            # files and directories with no hashes
+            self.assertEqual(
+                rmdir_calls, ['/srv/node/sda/containers/6789/jkl',
+                              '/srv/node/sda/containers/9876/xyz/'
+                              '11111111111111111111111111111xyz',
+                              '/srv/node/sdb/containers/6789/jkl',
+                              '/srv/node/sdb/containers/9876/xyz/'
+                              '11111111111111111111111111111xyz'])
+        finally:
+            db_replicator.os.listdir = orig_listdir
+            db_replicator.os.path.isdir = orig_isdir
+            db_replicator.os.path.exists = orig_exists
+            db_replicator.random.shuffle = orig_shuffle
+            db_replicator.os.rmdir = orig_rmdir
+
     def test_roundrobin_datadirs(self):
         listdir_calls = []
         isdir_calls = []
