@@ -44,9 +44,10 @@ class FakeReplicator(object):
 
 
 def write_diskfile(df, timestamp, data='test data', frag_index=None,
-                   commit=True, extra_metadata=None):
+                   commit=True, legacy_durable=False, extra_metadata=None):
     # Helper method to write some data and metadata to a diskfile.
-    # Optionally do not commit the diskfile
+    # Optionally do not commit the diskfile, or commit but using a legacy
+    # durable file
     with df.create() as writer:
         writer.write(data)
         metadata = {
@@ -59,7 +60,13 @@ def write_diskfile(df, timestamp, data='test data', frag_index=None,
         if frag_index is not None:
             metadata['X-Object-Sysmeta-Ec-Frag-Index'] = str(frag_index)
         writer.put(metadata)
-        if commit:
+        if commit and legacy_durable:
+            # simulate legacy .durable file creation
+            durable_file = os.path.join(df._datadir,
+                                        timestamp.internal + '.durable')
+            with open(durable_file, 'wb'):
+                pass
+        elif commit:
             writer.commit(timestamp)
         # else: don't make it durable
     return metadata
@@ -89,7 +96,6 @@ class BaseTest(unittest.TestCase):
             frag_index=frag_index)
         write_diskfile(df, timestamp, data=body, extra_metadata=extra_metadata,
                        commit=commit)
-
         if commit:
             # when we write and commit stub data, sanity check it's readable
             # and not quarantined because of any validation check
