@@ -28,7 +28,6 @@ from time import time
 from distutils.dir_util import mkpath
 
 from eventlet import spawn, Timeout, listen
-from six.moves import range
 
 from swift.obj import updater as object_updater
 from swift.obj.diskfile import (ASYNCDIR_BASE, get_async_dir, DiskFileManager,
@@ -78,20 +77,20 @@ class TestObjectUpdater(unittest.TestCase):
         rmtree(self.testdir, ignore_errors=1)
 
     def test_creation(self):
-        cu = object_updater.ObjectUpdater({
+        ou = object_updater.ObjectUpdater({
             'devices': self.devices_dir,
             'mount_check': 'false',
             'swift_dir': self.testdir,
             'interval': '1',
             'concurrency': '2',
             'node_timeout': '5.5'})
-        self.assertTrue(hasattr(cu, 'logger'))
-        self.assertTrue(cu.logger is not None)
-        self.assertEqual(cu.devices, self.devices_dir)
-        self.assertEqual(cu.interval, 1)
-        self.assertEqual(cu.concurrency, 2)
-        self.assertEqual(cu.node_timeout, 5.5)
-        self.assertTrue(cu.get_container_ring() is not None)
+        self.assertTrue(hasattr(ou, 'logger'))
+        self.assertTrue(ou.logger is not None)
+        self.assertEqual(ou.devices, self.devices_dir)
+        self.assertEqual(ou.interval, 1)
+        self.assertEqual(ou.concurrency, 2)
+        self.assertEqual(ou.node_timeout, 5.5)
+        self.assertTrue(ou.get_container_ring() is not None)
 
     @mock.patch('os.listdir')
     def test_listdir_with_exception(self, mock_listdir):
@@ -174,15 +173,15 @@ class TestObjectUpdater(unittest.TestCase):
                     seen.add((update_path, int(policy)))
                     os.unlink(update_path)
 
-            cu = MockObjectUpdater({
+            ou = MockObjectUpdater({
                 'devices': self.devices_dir,
                 'mount_check': 'false',
                 'swift_dir': self.testdir,
                 'interval': '1',
                 'concurrency': '1',
                 'node_timeout': '5'})
-            cu.logger = mock_logger = mock.MagicMock()
-            cu.object_sweep(self.sda1)
+            ou.logger = mock_logger = mock.MagicMock()
+            ou.object_sweep(self.sda1)
             self.assertEqual(mock_logger.warning.call_count, warn)
             self.assertTrue(
                 os.path.exists(os.path.join(self.sda1, 'not_a_dir')))
@@ -210,22 +209,22 @@ class TestObjectUpdater(unittest.TestCase):
     @mock.patch.object(object_updater, 'ismount')
     def test_run_once_with_disk_unmounted(self, mock_ismount):
         mock_ismount.return_value = False
-        cu = object_updater.ObjectUpdater({
+        ou = object_updater.ObjectUpdater({
             'devices': self.devices_dir,
             'mount_check': 'false',
             'swift_dir': self.testdir,
             'interval': '1',
             'concurrency': '1',
             'node_timeout': '15'})
-        cu.run_once()
+        ou.run_once()
         async_dir = os.path.join(self.sda1, get_async_dir(POLICIES[0]))
         os.mkdir(async_dir)
-        cu.run_once()
+        ou.run_once()
         self.assertTrue(os.path.exists(async_dir))
         # mount_check == False means no call to ismount
         self.assertEqual([], mock_ismount.mock_calls)
 
-        cu = object_updater.ObjectUpdater({
+        ou = object_updater.ObjectUpdater({
             'devices': self.devices_dir,
             'mount_check': 'TrUe',
             'swift_dir': self.testdir,
@@ -235,34 +234,34 @@ class TestObjectUpdater(unittest.TestCase):
         odd_dir = os.path.join(async_dir, 'not really supposed '
                                'to be here')
         os.mkdir(odd_dir)
-        cu.run_once()
+        ou.run_once()
         self.assertTrue(os.path.exists(async_dir))
         self.assertTrue(os.path.exists(odd_dir))  # skipped - not mounted!
         # mount_check == True means ismount was checked
         self.assertEqual([
             mock.call(self.sda1),
         ], mock_ismount.mock_calls)
-        self.assertEqual(cu.logger.get_increment_counts(), {'errors': 1})
+        self.assertEqual(ou.logger.get_increment_counts(), {'errors': 1})
 
     @mock.patch.object(object_updater, 'ismount')
     def test_run_once(self, mock_ismount):
         mock_ismount.return_value = True
-        cu = object_updater.ObjectUpdater({
+        ou = object_updater.ObjectUpdater({
             'devices': self.devices_dir,
             'mount_check': 'false',
             'swift_dir': self.testdir,
             'interval': '1',
             'concurrency': '1',
             'node_timeout': '15'}, logger=self.logger)
-        cu.run_once()
+        ou.run_once()
         async_dir = os.path.join(self.sda1, get_async_dir(POLICIES[0]))
         os.mkdir(async_dir)
-        cu.run_once()
+        ou.run_once()
         self.assertTrue(os.path.exists(async_dir))
         # mount_check == False means no call to ismount
         self.assertEqual([], mock_ismount.mock_calls)
 
-        cu = object_updater.ObjectUpdater({
+        ou = object_updater.ObjectUpdater({
             'devices': self.devices_dir,
             'mount_check': 'TrUe',
             'swift_dir': self.testdir,
@@ -272,7 +271,7 @@ class TestObjectUpdater(unittest.TestCase):
         odd_dir = os.path.join(async_dir, 'not really supposed '
                                'to be here')
         os.mkdir(odd_dir)
-        cu.run_once()
+        ou.run_once()
         self.assertTrue(os.path.exists(async_dir))
         self.assertTrue(not os.path.exists(odd_dir))
         # mount_check == True means ismount was checked
@@ -297,10 +296,10 @@ class TestObjectUpdater(unittest.TestCase):
                                  'X-Container-Timestamp':
                                  normalize_timestamp(0)}},
                             async_pending)
-        cu.run_once()
+        ou.run_once()
         self.assertTrue(not os.path.exists(older_op_path))
         self.assertTrue(os.path.exists(op_path))
-        self.assertEqual(cu.logger.get_increment_counts(),
+        self.assertEqual(ou.logger.get_increment_counts(),
                          {'failures': 1, 'unlinks': 1})
         self.assertIsNone(pickle.load(open(op_path)).get('successes'))
 
@@ -330,14 +329,13 @@ class TestObjectUpdater(unittest.TestCase):
             return None
 
         def accept(return_codes):
-            codes = iter(return_codes)
             try:
                 events = []
-                for x in range(len(return_codes)):
+                for code in return_codes:
                     with Timeout(3):
                         sock, addr = bindsock.accept()
                         events.append(
-                            spawn(accepter, sock, next(codes)))
+                            spawn(accepter, sock, code))
                 for event in events:
                     err = event.wait()
                     if err:
@@ -347,41 +345,41 @@ class TestObjectUpdater(unittest.TestCase):
             return None
 
         event = spawn(accept, [201, 500, 500])
-        for dev in cu.get_container_ring().devs:
+        for dev in ou.get_container_ring().devs:
             if dev is not None:
                 dev['port'] = bindsock.getsockname()[1]
 
-        cu.logger._clear()
-        cu.run_once()
+        ou.logger._clear()
+        ou.run_once()
         err = event.wait()
         if err:
             raise err
         self.assertTrue(os.path.exists(op_path))
-        self.assertEqual(cu.logger.get_increment_counts(),
+        self.assertEqual(ou.logger.get_increment_counts(),
                          {'failures': 1})
         self.assertEqual([0],
                          pickle.load(open(op_path)).get('successes'))
 
-        event = spawn(accept, [404, 500])
-        cu.logger._clear()
-        cu.run_once()
+        event = spawn(accept, [404, 201])
+        ou.logger._clear()
+        ou.run_once()
         err = event.wait()
         if err:
             raise err
         self.assertTrue(os.path.exists(op_path))
-        self.assertEqual(cu.logger.get_increment_counts(),
+        self.assertEqual(ou.logger.get_increment_counts(),
                          {'failures': 1})
-        self.assertEqual([0, 1],
+        self.assertEqual([0, 2],
                          pickle.load(open(op_path)).get('successes'))
 
         event = spawn(accept, [201])
-        cu.logger._clear()
-        cu.run_once()
+        ou.logger._clear()
+        ou.run_once()
         err = event.wait()
         if err:
             raise err
         self.assertTrue(not os.path.exists(op_path))
-        self.assertEqual(cu.logger.get_increment_counts(),
+        self.assertEqual(ou.logger.get_increment_counts(),
                          {'unlinks': 1, 'successes': 1})
 
     def test_obj_put_legacy_updates(self):
